package dsql;

import dsql.exception.*;
import dsql.parser.ASTConnect;
import dsql.parser.ASTDriver;
import dsql.parser.ASTDBMS;

import java.sql.*;

/** Creates a {@link dsql.Connect Connect} object from a 
 * &lt;connect&gt; element */
public class ConnectFactory {

  public static Connect getConnectObject(ASTConnect astc) 
      throws ExecuteException {
    ASTDBMS dbms = astc.getDBMS();
    ASTDriver driver = null;
    String connectionStr = null;
    if(astc.getDriver()!=null) {
      driver = astc.getDriver();
      connectionStr = driver.getConnectionString(astc);
    } else {
      ASTDriver[] drivers = dbms.getDrivers();
      for(int i=0; i<drivers.length && driver==null; i++)
        try {
          connectionStr = drivers[i].getConnectionString(astc);
          driver = drivers[i];
        } catch (ConnectStringException exc) {}
      if(driver==null) throw new ExecuteException("no driver matches "+
        "parameters of <connect id=\""+astc.getId()+"\">");
    }
    return new Connect(astc.getId(),driver,
                       connectionStr,astc.getUserName(),astc.getPassword());
  }


}
