package dsql;

import dsql.exception.*;

import java.io.*;

/** Utility class to output trace/debug messages. */

public class Trace {

  protected static final int NOWARNING = 0;
  protected static final int QUIET = 1;
  protected static final int SQL = 2;
  protected static final int TRACE = 3;
  protected static final int DEBUG = 4;

  protected static final String[] TRACESTR = {"nowarning",
    "no","sql","yes","debug"};

  protected PrintStream out = System.err;
  protected int mode = QUIET;

  public Trace() {}
  public Trace(PrintStream _out) {out=_out;}
  public Trace(String _modeStr) throws ExecuteException {
    if(_modeStr!=null) set(_modeStr);
  }
  public Trace(PrintStream _out, String _modeStr) throws ExecuteException {
    this(_out);
    if(_modeStr!=null) set(_modeStr);
  }

  public void set(String _modeStr) throws ExecuteException {
    setMode(getMode(_modeStr));
  }

  public void setMode(int _mode) {
    mode = _mode;
  }
  public int getMode() {return mode;}

  public int getMode(String _modeStr) throws ExecuteException {
    int newMode = -1;
    for(int i=0; i<TRACESTR.length; i++) {
      if(_modeStr.equalsIgnoreCase(TRACESTR[i])) newMode = i;
    }
    if(newMode==-1)
      throw new ExecuteException("Invalid trace mode: "+_modeStr);
    else
      return newMode;
  }

  public boolean issql() {return mode>=SQL;}
  public boolean istrace() {return mode>=TRACE;}
  public boolean isdebug() {return mode>=DEBUG;}

  public void sql(String msg) {
    if(issql()) out.println(msg);    
  }
  public void trace(String msg) {
    if(istrace()) out.println(msg);    
  }
  public void debug(String msg) {
    if(isdebug()) out.println(msg);    
  }
  public void warning(String msg) {
    if(mode>NOWARNING) out.println("*** Warning: "+msg);
  }

  public String maxstr(String msg) {return maxstr(msg,80);}
  public String maxstr(String msg,int max) {
    if(msg.length()>max) return msg.substring(0,max-3)+"...";
    else return msg;
  }

}
