package dsql.parser;

import dsql.ConnectFactory;
import dsql.exception.*;
import dsql.value.*;

/** A <connect> element */
public class ASTConnect extends ASTDocContext {

  /** the DBMS object corresponding to dbmsid
   * will only be set after execute() */
  ASTDBMS dbms = null;  

  /** this is only set if driverid attribute was
   * specified */
  ASTDriver driver = null; 

  /** Constructor for Parser */
  public ASTConnect(int id) {
    super(id);
  }

  /** Constructor for Parser */
  public ASTConnect(Parser p, int id) {
    super(p, id);
  }

  /** Constructor for ASTXMLElement.recast() */
  public ASTConnect(ASTXMLElement toCopy) {
    super(toCopy);
  }

  public String toString() {
    String driverid=getDriverId();
    return "<connect id=\""+getId()+"\" dbmsid=\""+getDBMSId()+"\""+
	(driverid!=null?" driverid=\""+driverid+"\"":"")+">";
  }

  public String getDBMSId() {
    return getAttributeValue("dbmsid");
  }

  public String getDriverId() {
    return getAttributeValue("driverid");
  }

  public String getId() {
    String id = getAttributeValue("id");
    if(id==null) return "default"; else return id;
  }

  public ASTUserName getUserName() {
    ASTUserName uname = (ASTUserName) getElement(ASTUserName.class);
    if(uname==null) uname = new ASTUserName(-1);
    return uname;
  }
  public ASTPassword getPassword() throws ExecuteException {
    ASTPassword pwd = (ASTPassword) getElement(ASTPassword.class);
    if(pwd==null) pwd = new ASTPassword(-1);
    else
	if(pwd.getPassword().length()>0 && !pwd.isEncrypted()) {
        System.err.println("* Warning: the password in <connect id=\""+getId()+
                           "\"> is not encrypted!");
        try {
          System.err.println("  Encrypt it with: <password encrypt=\"dsql\">"+
                             ASTPassword.encrypt(pwd.getXMLText(),"dsql")+
                             "</password>");
        } catch (ExecuteException e) {
          System.err.println("* Unable to encrypt password: "+e.getMessage());
        }
      }
    return pwd;
  }

  public ASTDBMS getDBMS() {return dbms;}
  public ASTDriver getDriver() {return driver;}

  public Value execute() throws ExecuteException {
    ASTConnect resThis = (ASTConnect) executeXML();
    //    if(dbms==null) 
    String dbmsId = resThis.getDBMSId();
    if(dbmsId==null)
      throw new ExecuteException("* Element <connect id=\""+
        getId()+"\"> does not have a dbmsid attribute");
    resThis.dbms = resThis.getDoc().getDBMS(dbmsId);
    if(resThis.dbms==null)
	throw new ExecuteException("* Unknown DBMS: '"+dbmsId+"'");
    if(resThis.getDriverId()!=null)
      resThis.driver = dbms.getDriver(resThis.getDriverId());
    resThis.getDoc().addConnect(ConnectFactory.getConnectObject(resThis));
    return new XmlDataValue(resThis);
  }

}
