package dsql.parser;

import dsql.exception.*;
import dsql.value.*;

import java.lang.reflect.*;
import java.io.*;

/** A data element, e.g. XML or Excel. When executed, this object
 * returns a DataValue. New types of data can be added by subclassing
 * DataValue, e.g. ExcelDataValue.  */
public class ASTData extends ASTXMLElement {

  /** Constructor for Parser */
  public ASTData(int id) {
    super(id);
  }

  /** Constructor for Parser */
  public ASTData(Parser p, int id) {
    super(p, id);
  }

  /** Constructor for Parser recast */
  public ASTData(ASTXMLElement toCopy) {
    super(toCopy);
  }

  public String toString() {
    String file = null, type = null;
    try {file = getFile(); type = getType(file);} 
    catch (ExecuteException exc) {}
    return "<data type=\""+type+"\" file=\""+file+"\">";
  }

  String file; // make sure getFile() gets evaluated just once,
               // in case there is some dsql code in element...
  protected String getFile() throws ExecuteException {
    String file = getAttributeValue("file");
    if(file==null) file="inline";
    return file;
  }

  public String getType(String file) throws ExecuteException {
    String type = getAttributeValue("type");
    if(type==null) {
      if(file!=null && file.endsWith(".xls")) return "excel";
      if(file!=null && file.endsWith(".xml")) return "xml";
      return "xml"; 
    }
    return type;
  }

  public Value execute() throws ExecuteException {
    ASTData xmldata = (ASTData) executeXML();
    Class inputClass = null;
    String file = xmldata.getFile();
    String type = xmldata.getType(file);
    if(type.length()>0)
      type = type.substring(0,1).toUpperCase() +
	  ((type.length()>1) ? type.substring(1).toLowerCase() : "");
    // find class
    try {
      String className = "dsql.value."+type+"DataValue";
      inputClass = Class.forName(className);
    } catch (ClassNotFoundException e) {
        throw new ExecuteException("Data type \""+type+
          "\" not supported");
    }
    // TBD: should reuse code in ASTNew
    // find class constructor
    Constructor cons = null;
    try {
	//Class[] consTypes = new Class[0];
      cons = inputClass.getConstructor(null);
    } catch (NoSuchMethodException e) {
      throw new ExecuteException("Class "+
        inputClass.getName()+" does not have an appropriate constructor");
    }
    // instantiate
    //Object[] consArgs = new Object[0];
    try {
      DataValue data = (DataValue) cons.newInstance(null);
      if(file.equalsIgnoreCase("inline"))
        data.setContents(xmldata);
      else
	data.setContents(file,xmldata);      
      return (Value) data;
    } catch (InstantiationException e) {
      throw new ExecuteException("Unable to instantiate class "+
                                 inputClass.getName()+": "+e.getMessage());
    } catch (IllegalAccessException e) {
      throw new ExecuteException("Unable to instantiate class "+
                                 inputClass.getName()+": "+e.getMessage());
    } catch (InvocationTargetException e) {
      if(e.getCause() instanceof ExecuteException) 
        throw (ExecuteException) e.getCause();
      else throw new ExecuteException(e.getCause().getMessage());
    }
  }

}
