package dsql.parser;

import dsql.Connect;
import dsql.Variable;
import dsql.exception.*;
import dsql.value.*;

import java.io.*;
import java.util.*;

/** A document element */
public class ASTDoc extends ASTDsql {

  String fileName = null;
  boolean included = false;

  Hashtable connects = new Hashtable(), // table of <connect> elements
    dbms = new Hashtable(),             // table of <dbms> elements
    subs = new Hashtable();
  String defaultConnectId = "default";
    // name of the first <connect> element encountered in the document
    // or "default"
  Vector vars = new Vector(); // list of variable context
  String[] args = null;
  static String[] ARGS_LABELS = {"arg"};

  /** Constructor for Parser */
  public ASTDoc(int id) {
    super(id);
  }

  /** Constructor for Parser */
  public ASTDoc(Parser p, int id) {
    super(p, id);
  }

  public void setFileName(String name) {fileName=name;}
  public String getFileName() {return fileName;}
  public void setIncluded() {included=true;}
  public boolean isIncluded() {return included;}

  public String toString() {return "<"+getTagName()+" file=\""+
                                   getFileName()+"\">";}

  public void setArgs(String[] _args) {
    args = _args;
  }

  public Value execute() throws ExecuteException {
    if(vars.isEmpty()) pushVariableContext();
    if(args!=null) {
	dsql.value.List ct = new dsql.value.List(getCaller(),args.length);
      for(int i=0; i<args.length; i++)
	  ct.getList().add(args[i]);
      addVariable(new Variable(getCaller(),"ARGS",ct));
    }
    if(getFileName()!=null) {
      File file = new File(new File(getFileName()).getAbsolutePath());
      String fs = file.getParent();
      if(fs.endsWith(".")) fs = fs.substring(0,fs.length()-1);
      if(!fs.endsWith("/") && !fs.endsWith("\\"))
	fs = fs+File.separator; 
      addVariable(new Variable(caller,"FILEPATH",new StringValue(caller,fs)));
    }
    Value v= super.execute();
    //    if(v!=null) caller.trace.warning("* Ignored value: "+v);
    return v;
  }

  // Variables

  public Variable getVariable(String name) throws ExecuteException {
    for(int i=vars.size()-1; i>=0; i--) {
      Hashtable varTable = (Hashtable) vars.elementAt(i);
      Variable var = (Variable) varTable.get(name);
      if(var!=null) return var;
    }
    throw new ExecuteException("Variable $"+name+" is not set");
  }

  /** Adds a variable to the current context */
  public void addVariable(Variable var) {
    ((Hashtable)vars.elementAt(vars.size()-1)).put(var.getName(),var);
    caller.trace.debug("Added "+var+" to the current variable context");
  }

  /** Sets a variable: in the closest context if it exists, or adds
   * otherwise */
  public void setVariable(String name, Value value) {
    Variable var;
    try { var = getVariable(name); }
    catch (ExecuteException exc) { 
      var = new Variable(caller,name);
      addVariable(var); 
    }
    var.setValue(value);
  }

  public void pushVariableContext() {
    vars.add(new Hashtable());
    caller.trace.debug("Pushed variable context");
  }

  public void popVariableContext() throws ExecuteException {
    int lastContext = vars.size()-1;
    if(lastContext==0) throw new ExecuteException("Can't pop document "+
                                                  "variable context");
                       // this should never happen!
    vars.removeElementAt(lastContext);
    caller.trace.debug("Popped variable context");
  }

  // DBMS

  public void addDBMS(ASTDBMS adbms) {
    dbms.put(adbms.getId(),adbms);
  }

  public ASTDBMS getDBMS(String dName) throws ExecuteException {
    ASTDBMS d = (ASTDBMS) dbms.get(dName);
    if(d==null) throw new ExecuteException("No <dbms> element with id=\""+
                                           dName+"\"");
    return d;
  }

  // CONNECT

  public void addConnect(Connect connect) {
    if(connects.isEmpty() || connect.getId().equals("default")) 
      defaultConnectId = connect.getId();
    connects.put(connect.getId(),connect);
  }

  /** @return the &lt;connect&gt; element with the corresponding
   * name */
  public Connect getConnect(String cName) throws ExecuteException {
    if(cName.equals("default"))
      cName = defaultConnectId;
    if(!connects.containsKey(cName))
      throw new ExecuteException("No <connect> element with id=\""+
		     cName+"\"");
    return (Connect) connects.get(cName);
  }

  // Subroutines

  public void addSub(ASTSub sub) throws ExecuteException {
    if(subs.containsKey(sub.getName()))
       throw new ExecuteException("Subroutine "+sub.getName()+
				  " already exists");
    subs.put(sub.getName(),sub);
  }

  public ASTSub getSub(String name) {
    return (ASTSub) subs.get(name);
  }

}
