package dsql.parser;

import dsql.exception.*;
import dsql.value.*;

/** A dot construct, e.g. $x.FirstName */
public class ASTDot extends DMNode implements Assignable {

    /** true for ".@" syntax, eg $x.@4 to get fourth column */
  boolean hash = false;
 
  /** Constructor for the parser */
  public ASTDot(int id) {
    super(id);
  }

  /** Constructor for the parser */
  public ASTDot(Parser p, int id) {
    super(p, id);
  }

  public void setHash() {hash=true;}

  public String toString() {return ".";}

  public Value execute() throws ExecuteException {
    Value obj = executeArgumentNonNull(0);
    String column = executeArgumentNonNull(1).getString();
    Object val;
    if(hash) val = getDotAt(obj).getField(getColumnInt(column));
    else val = obj.getField(column);
    return Value.getValue(getCaller(),val);
  }

  public void setValue(Value to) throws ExecuteException {
    Value obj = executeArgumentNonNull(0);
    String column = executeArgumentNonNull(1).getString();
    if(hash) getDotAt(obj).setField(getColumnInt(column),to);
    else obj.setField(column,to);
  }

  // ********************************

  protected DotAt getDotAt(Value obj) throws ExecuteException {
    if(obj instanceof DotAt)
      return (DotAt) obj;
    else 
      throw new ExecuteException(obj+" does not support .@");
  }

  protected int getColumnInt(String column) throws ExecuteException {
    try {return Integer.parseInt(column);}
    catch (NumberFormatException exc) {throw new ExecuteException(
		      "Invalid integer number: "+exc.getMessage(),exc);}
  }

}
