package dsql.parser;

import dsql.exception.*;


import dsql.value.*;

/** In operator: e.g $x in $y. can check for scalar values,
 * records or tables */
public class ASTInOp extends ASTSetOperator {

  public ASTInOp(int id) {
    super(id);
  }

  public ASTInOp(Parser p, int id) {
    super(p, id);
  }

  public Value execute() throws ExecuteException {
    Value v1 = executeArgumentNonNull(0);
    if(v1.isScalar()) 
      return executeScalar(v1.getString());
    else if(v1 instanceof TableResult)
      return executeTable((TableResult) v1);
    else if((v1 instanceof ColumnHolder))
      return executeRecord((ColumnHolder) v1);
    throw new ExecuteException("Invalid operand for in operator: "+v1);
  }

  protected Value executeScalar(String v1s) throws ExecuteException {
    TableResult t2 = executeArgumentTable(1);
    t2.reset();
    while(t2.nextRecord())
      if(t2.getString().equals(v1s)) return BooleanValue.TRUE;
    return BooleanValue.FALSE;
  }


  protected Value executeRecord(ColumnHolder rec) throws ExecuteException {
    int colcount = rec.getColumnCount();
    TableResult t2 = executeArgumentTable(1);
    t2.reset();
    while(t2.nextRecord()) {
      if(t2.equalsRecord(rec)) return BooleanValue.TRUE;
      //RecordHash rechash = tref.getNewRecordHash(t2.readCurrentRecord());
    }
    return BooleanValue.FALSE;
  }


  protected Value executeTable(TableResult t1) throws ExecuteException {
    TableResult t2 = executeArgumentTable(1);
    ConstructedTableHash t2h = new ConstructedTableHash(t2);
    t2h.insert(t2);
    t1.reset();
    while(t1.nextRecord()) {
      RecordHash rechash = t2h.getNewRecordHash(t1.readCurrentRecord());
      if(!t2h.exists(rechash)) {return BooleanValue.FALSE;}
    }
    return BooleanValue.TRUE;
  }

}
