package dsql.parser;

import dsql.exception.*;


import dsql.value.*;

import java.io.*;

/** An <include> element; e.g. <include file="mydbms.dsql"/>. */
public class ASTInclude extends ASTDocContext {

  /** Constructor for Parser */
  public ASTInclude(int id) {
    super(id);
  }

  /** Constructor for Parser */
  public ASTInclude(Parser p, int id) {
    super(p, id);
  }

  /** Constructor for ASTXMLElement.recast() */
  public ASTInclude(ASTXMLElement toCopy) {
    super(toCopy);
  }

  public ASTInclude(String fileName, ASTDoc doc) {
    super(-1);
    setTagName("include");
    setAttribute("file",fileName);
    setDoc(doc);
    setCaller(doc.getCaller());
  }

  public String toString() {
    String file = null;
    try {file = getFileName();}
    catch (ExecuteException exc) {}
    return "<"+getTagName()+(file!=null?" file=\""+file+"\"":"")+">";
  }

  public String getFileName() throws ExecuteException {
    return getAttributeValue("file");
  }

  public Value execute() throws ExecuteException {
    ASTDoc thisdoc = parse();
    getCaller().trace.debug("*** Executing...");
    if(getCaller().hasRequestToStop()) throw new StopException();
    return thisdoc.execute();
  }

  // *********** PROTECTED **********

  static String IncludePath = null;
  static String[] IncludeDirs = null;

  static void setIncludePath() {
    String DSQL_INCLUDEPATH = System.getProperty("DSQL_INCLUDE");
    if(DSQL_INCLUDEPATH!=null && !DSQL_INCLUDEPATH.equals("")) 
       IncludePath = DSQL_INCLUDEPATH;
    else {
      String DSQL_HOME = System.getProperty("DSQL_HOME");
      if(DSQL_HOME!=null && !DSQL_HOME.equals("")) 
        IncludePath = DSQL_HOME+File.separator+"scripts"+
          File.pathSeparator+".";
      else IncludePath = ".";
    }
    IncludeDirs = IncludePath.split(File.pathSeparator,0);
  }
 
  public static Reader getFile(String fileName) throws ExecuteException {
    BufferedReader file = tryToOpenFile(fileName);
    if(file!=null) return file;
    if(IncludePath==null) setIncludePath();
    for(int i=0; i<IncludeDirs.length; i++) {
      file = tryToOpenFile(IncludeDirs[i]+File.separator+fileName); 
      if(file!=null) return file;
        /* OBSOLETE: now in Parser.jjt
         f.mark(256);
        if(f.readLine().startsWith("#")){
        } else {
          f.reset();
	  }*/
    }
    throw new ExecuteException("Cannot find file \""+fileName+"\" in \""+
                               IncludePath+"\"");
  }

  protected static BufferedReader tryToOpenFile(String path) 
    throws ExecuteException {
    try {
      if(new File(path).isFile())
        return new BufferedReader(new FileReader(path)); 
      else
        return null;
        /* OBSOLETE: now in Parser.jjt
         f.mark(256);
        if(f.readLine().startsWith("#")){
        } else {
          f.reset();
	  }*/
    } catch (IOException e) {
      throw new ExecuteException("Error opening \""+
				 path+"\":"+e.getMessage());
    }
  }

  protected ASTDoc parse() throws ExecuteException {
    String fileName = getFileName();
    Parser parser = new Parser(getFile(fileName));
    parser.setCaller(caller);
    getCaller().trace.debug("*** Parsing "+fileName+"...");
    try {
      ASTDoc thisdoc = parser.DocumentWithParent(getDoc(),fileName);
      //if(getCaller().trace.isdebug()) thisdoc.dump("");
      return thisdoc;
    } catch (ParseException exc) {
      throw new ExecuteException(fileName,exc);
    } catch (TokenMgrError exc) {
      throw new ExecuteException(fileName,exc);
    }
  }
  

}
