package dsql.parser;

import dsql.exception.*;
import dsql.value.*;

import java.lang.reflect.*;

/** Access to Java objects: constructor and method invoker */
public abstract class ASTJava extends DMNode {

  /** Constructor for Parser */
  public ASTJava(int id) {
    super(id);
  }

  /** Constructor for Parser */
  public ASTJava(Parser p, int id) {
    super(p, id);
  }

  /** Execute arguments */
  protected Value[] executeParameters(int first) throws ExecuteException {
    int nb = jjtGetNumChildren();
    if(first<nb) {
      Value[] pars = new Value[nb-first];
      for(int i=first; i<nb; i++)
        pars[i-first] = executeArgumentNonNull(i);
      return pars;
    } else 
      return null;
  }

  /** Converts arguments to basic Java objects */
  protected Object[] parsAsObjects(Value[] args) {
    if(args==null) return null;
    Object[] objs = new Object[args.length];
    for(int i=0; i<args.length; i++)
      objs[i] = args[i].getObject();
    return objs;
  }

  protected Class[] getParametersClasses(Value[] args)
      throws ExecuteException {
    if(args==null) return null;
    Class[] types = new Class[args.length];
    for(int i=0; i<args.length; i++) 
      types[i] = args[i].getObjectClass() ;
    return types;
  }

  protected String typesToString(Class[] types) {
    StringBuffer s = new StringBuffer("(");
    for(int i=0; i<types.length; i++) {
      if(i>0) s.append(",");
      s.append(types[i].getName());
    }
    s.append(")");
    return s.toString();
  }

}
