package dsql.parser;

import dsql.exception.*;
import dsql.value.*;

import java.lang.reflect.*;

// TBD: cache method object for efficiency
/** Method call,  eg "$x.set(x)" */
public class ASTMethodCall extends ASTJava {

  String methodName;

  /** Constructor for Parser */
  public ASTMethodCall(int id) {
    super(id);
  }

  /** Constructor for Parser */
  public ASTMethodCall(Parser p, int id) {
    super(p, id);
  }

  public Value execute() throws ExecuteException {
    Object object = executeArgumentNonNull(0).getObject();
    String methodName = executeArgumentNonNull(1).getString();
    Value[] pars = executeParameters(2);
    Class[] types = getParametersClasses(pars);
    try {
      Method meth = object.getClass().getMethod(methodName,types);
      Object r = meth.invoke(object,parsAsObjects(pars));
      return Value.getValue(getCaller(),r,meth.getReturnType());
    } catch (NoSuchMethodException exc) {
      String msg;
      Method[] methods = object.getClass().getMethods();
      if(methods.length>0) {
        StringBuffer nomatch = new StringBuffer(), match = new StringBuffer();
        for(int i=1; i<methods.length; i++) 
          if(methods[i].getName().equals(methodName))
            match.append(", "+methods[i].toString());
          else
            nomatch.append(", "+methods[i].getName());
        if(match.length()>0) {
          msg = "method "+object.getClass().getName()+"."+methodName+
            typesToString(types)+" does not match class "+
	    "definition: "+match.toString().substring(2);
        } else
          msg = "Method "+methodName+" is unknown. Try one of:"+
	    nomatch.toString().substring(2)+".";
      } else
  	  // this is probably impossible?
          msg = "No methods are available for this object.";
      throw new ExecuteException(msg);
    } catch (IllegalAccessException exc) {
      throw new ExecuteException("Method '"+methodName+
        "' cannot be accessed",exc);
    } catch (InvocationTargetException exc) {
      throw new ExecuteException("Error while invoking method "+methodName,exc);
    }
  }

  
}
