package dsql.parser;

import dsql.exception.*;
import dsql.value.*;

import java.lang.reflect.*;

/** Object creator, eg "new java.util.Vector()" */
public class ASTNew extends ASTJava {

  String className;

  /** Constructor for Parser */
  public ASTNew(int id) {
    super(id);
  }

  /** Constructor for Parser */
  public ASTNew(Parser p, int id) {
    super(p, id);
  }

  public String getName() {return className;}
  public void setName(String _className) {
    className = _className;
  }

  public Value execute() throws ExecuteException {
    return Value.getValue(getCaller(),newInstance(executeParameters(0)));
  }

  protected Object newInstance(Value[] args) throws ExecuteException {
    try {
      return getConstructor(getParametersClasses(args))
        .newInstance(parsAsObjects(args));
    } catch (InstantiationException e) {
      throw new ExecuteException("Unable to instantiate class "+
                                 getName()+": "+e.getMessage());
    } catch (IllegalAccessException e) {
      throw new ExecuteException("Unable to instantiate class "+
                                 getName()+": "+e.getMessage());
    } catch (InvocationTargetException e) {
      if(e.getCause() instanceof ExecuteException) 
        throw (ExecuteException) e.getCause();
      else throw new ExecuteException(e.getCause().getMessage());
    }
  }

  protected Constructor getConstructor(Class[] types) throws ExecuteException {
    try {
      return getClassObject().getConstructor(types);
    } catch (NoSuchMethodException e) {
      throw new ExecuteException("Constructor "+getName()+typesToString(types)+
				 " does not match class definition: "+
				 getConstructorsDefinition());
    }
  }

  protected Class getClassObject() throws ExecuteException {
    try {
      return Class.forName(getName());
    } catch (ClassNotFoundException e) {
	throw new ExecuteException("Java class "+getName()+" not found");
    }
  }

  protected String getConstructorsDefinition() throws ExecuteException {
    StringBuffer d = new StringBuffer();
    Constructor[] c = getClassObject().getConstructors();
    for(int i=0; i<c.length; i++) {
      if(i>0) d.append(", ");
      d.append(c[i].toString());
    }
    return d.toString();
  }

}
