package dsql.parser;

import dsql.exception.*;

import dsql.Variable;
import dsql.VariableOutputStream;
import dsql.value.*;

import java.io.*;
import java.sql.*;
import java.util.regex.*;

// TBD: major clean up needed here to move this code to the classes */

/** A print instruction */
public class ASTPrint extends DMNode {

  PrintStream out;

  boolean println = false,  // print new line (println)
          all = false,      // print all results
          append = false,   // append to file
          tofile = false,   // print to file
          header = true;    // prints header line for tables
  int mode = 0; // 0: table; 1: csv; 2: xml

  public ASTPrint(int id) {
    super(id);
  }

  public ASTPrint(Parser p, int id) {
    super(p, id);
  }

  // TBD: remove!
  public int getMode() {return mode;}

  public boolean getHeader() {return header;}

  public void setAll() {
    all = true;
  }

  public void setPrintln() {
    println = true;
  }

  public void setTableMode() {
    mode = 0;
  }

  public void setCSVMode() {
    mode = 1;
  }

  public void setXMLMode() {
    mode = 2;
  }

  public void setAppend() {
    append = true;
  }

  public void setToFile() {
    tofile = true;
  }

  public void setHeader(boolean _header) {
    header = _header;
  }

  public Value execute() throws ExecuteException {
    Value r = executeArgumentNonNull(0);
    if(hasArgument(1)) {
      if(!tofile && getArgument(1) instanceof ASTVariable) {
        ASTVariable outastv = (ASTVariable) getArgument(1);
        String prevValue = null;
        Variable var = null;
        try {var = getRoot().getVariable(outastv.getName());}
        catch (ExecuteException exc) {}
        if(var==null)
          getRoot().addVariable(new Variable(caller,outastv.getName()));
        else if(append) 
          prevValue = var.getValue().getString();
        out = new PrintStream(new VariableOutputStream(getRoot().getVariable(
					     outastv.getName())));
	if(prevValue!=null) out.print(prevValue);
      } else {
        String fname = executeArgumentNonNull(1).getString();
        try {out = new PrintStream(new FileOutputStream(fname,append));}
        catch (FileNotFoundException exc) {throw new ExecuteException(
          exc.toString());}
      }
    } else
      // no output stream specified in statement, get it from enclosing <dsql>
      out = getParentOut();
    r.print(this);
    if(println) println();
    return null;
  }

  protected PrintStream getParentOut() {
    OutputStream _out = null;
    ASTDsql parent = getDsqlParent();
    while(parent!=null && (_out=parent.getOut())==null)
      parent = parent.getDsqlParent();
    if(_out==null) return System.out;
    if(_out instanceof PrintStream) return (PrintStream) _out;
    else return new PrintStream(_out);
  }

  // TBD: all this code should really go in table classes!

  /** Prints a multiple results object */
  public void printResults(AbstractResults r) throws ExecuteException {
    boolean more = r.hasResult();
    while(more) {
      printTable(r);
      if(all) more = r.nextResult();
      else more = false;
    }
  }

  public void printTable(TableResult r) throws ExecuteException {
    if(r.getColumnCount()>0) {
	if(mode==2) {println("<"+r.getTableName()+">");}
      else {printTableHeader(r);}
      r.reset();
      while(r.nextRecord())
        printRecord(r);
      if(mode==0 && header) printTableLine();
      else if(mode==2) println("</"+r.getTableName()+">");
    }
  }

  static String DASHES = "--------------------";
  static String BLANKS = "                    ";

  int[] colLength = null; // for print mode 0; display size of each column

  public void printTableHeader(ColumnHolder r) throws ExecuteException {
    int numCols = r.getColumnCount();
    if(mode==0) {
      // TABLE mode
      colLength = new int[numCols];
      int maxLength = BLANKS.length();
      for(int i=1; i<=numCols; i++) {
        int csize = r.getColumnDisplaySize(i);
        int lsize = r.getColumnLabel(i).length();
        if(lsize>csize) 
          // increase to label size, if longer
          csize = lsize;
        colLength[i-1] = csize;
        if(colLength[i-1]>maxLength) maxLength = csize;
      }
      if(maxLength>DASHES.length()) {
        StringBuffer s = new StringBuffer(DASHES);
        while (s.length()<maxLength) s.append('-');
        DASHES = s.toString();
      }
      if(maxLength>BLANKS.length()) {
        StringBuffer s = new StringBuffer(BLANKS);
        while (s.length()<maxLength) s.append(' ');
        BLANKS = s.toString();
      }
    } else if(mode==2) {
      println("<table>");
      return;
    }
    if(header) {
      if(mode==0) printTableLine();
      if(mode==0) print("|");
      for(int i=1; i<=numCols; i++)
        printColumn(i,r.getColumnLabel(i));
      if(mode==0) print("|");
      println();
      if(mode==0) printTableLine();
    }
  }


  public void printRecord(ColumnHolder r) throws ExecuteException {
    int numCols = r.getColumnCount();
    if(mode==0) print("|");
    else if(mode==2) print("<record>");
    for(int i=1; i<=numCols; i++) {
      if(mode==2) printXMLColumn(r.getColumnLabel(i),r.getColumnString(i));
      else printColumn(i,r.getColumnString(i));
    } 
    if(mode==0) print("|");
    else if(mode==2) print("</record>");
    println();
  }


  public void printXMLColumn(String label,String val) {
    print("<"+label+">");
    print(val.replaceAll("&","&amp;").replaceAll("<","&lt;").
          replaceAll(">","&gt;"));
    print("</"+label+">");
  }

  public void printColumn(int i,String val) {
    int maxSize = -1, vallength = -1;
    if(val==null) val="NULL";
//    if(val!=null) 
    vallength = val.length();
    if(mode==0 && colLength!=null)
      maxSize = colLength[i-1];
    if(maxSize!=-1 && vallength>maxSize)
      val = val.substring(0,maxSize);
    if(i>1) // separator
      if(mode==0) print("|");
      else if(mode==1) print(",");
    if(mode==1 && val!=null && (val.indexOf(',')!=-1 || 
       val.indexOf('\n')!=-1)) {
      val = "\""+val.replaceAll("\"","\"\"")+"\"";
    }
    print(val);
    if(maxSize!=-1 && vallength<maxSize)
      print(BLANKS.substring(0,maxSize-vallength));
  }


  /** Prints '+-----+-----+' line */
  public void printTableLine() {
    print("+");
    for(int i=0; i<colLength.length; i++)
      print(DASHES.substring(0,colLength[i])+"+");
    println();
  }

  public void print(String s) {
    out.print(s);
  }

  public void println() {println("");}
  public void println(String s) {
    out.println(s);
  }

}
