package dsql.parser;

import dsql.Connect;
import dsql.exception.*;
import dsql.value.*;

import java.sql.*;
import java.util.*;

/** An <sql> element */
public class ASTSQL extends ASTDocContext {

  Connect connect = null; // only set after execute()

  /** Constructor for Parser */
  public ASTSQL(int id) {
    super(id);
  }

  /** Constructor for Parser */
  public ASTSQL(Parser p, int id) {
    super(p, id);
  }

  /** Constructor for ASTXMLElement.recast() */
  public ASTSQL(ASTXMLElement toCopy) {
    super(toCopy);
  }

  public String getConnectId() {
    String id = getAttributeValue("connectid");
    if(id==null) return "default"; else return id;
  }

  public Connect getConnect() {return connect;}

  public String toString() {
    return "<sql connectid=\""+getConnectId()+"\">";
  }

  public String getPrimaryKeysAttribute() throws ExecuteException {
    return getAttributeValue("primaryKeys");
  }

  public String[] getPrimaryKeysFromAttribute(String keysStr) {
    if(keysStr==null) return null;
    return keysStr.split(",");
  }

  public String[] getPrimaryKeys() throws ExecuteException {
    return getPrimaryKeysFromAttribute(getPrimaryKeysAttribute());
  }

  public String getExecuteString() throws ExecuteException {
    return executeXML().getXMLText().trim();
  }


  public Value execute() throws ExecuteException {
    if(connect==null) connect = getDoc().getConnect(getConnectId());
    return getConnect().execute(this,getExecuteString());
  }

}
