package dsql.parser;

import dsql.exception.*;


import dsql.value.*;

import javax.mail.*;
import javax.mail.internet.*;
import java.util.*;

/** Sends an email using Sun JavaMail */
public class ASTSendEmail extends DMNode {

  public ASTSendEmail(int id) {
    super(id);
  }

  public ASTSendEmail(Parser p, int id) {
    super(p, id);
  }

  public Value execute() throws ExecuteException {
    Value tr = executeArgumentNonNull(0);
    String[] recipients = {
	Value.getValue(getCaller(),tr.getField("to")).getString()};
    try {
	postMail(Value.getValue(getCaller(),tr.getField("host")).getString(),recipients,
		 Value.getValue(getCaller(),tr.getField("subject")).getString(),
		 Value.getValue(getCaller(),tr.getField("message")).getString(),
		 Value.getValue(getCaller(),tr.getField("from")).getString());
      return null;
    } catch (MessagingException exc) {
	throw new ExecuteException(exc.getMessage());
    }
  }


  public void postMail(String host, String[] recipients, String subject, 
    String message, String from) throws MessagingException
  {
    boolean debug = false;

     //Set the host smtp address
     Properties props = new Properties();
     props.put("mail.smtp.host",host);

     // create some properties and get the default Session
    Session session = Session.getDefaultInstance(props, null);
    session.setDebug(debug);

    // create a message
    Message msg = new MimeMessage(session);

    // set the from and to address
    InternetAddress addressFrom = new InternetAddress(from);
    msg.setFrom(addressFrom);

    InternetAddress[] addressTo = new InternetAddress[recipients.length]; 
    for (int i = 0; i < recipients.length; i++)
    {
        addressTo[i] = new InternetAddress(recipients[i]);
    }
    msg.setRecipients(Message.RecipientType.TO, addressTo);
   

    // Optional : You can also set your custom headers in the Email if you Want
    //msg.addHeader("MyHeaderName", "myHeaderValue");

    // Setting the Subject and Content Type
    msg.setSubject(subject);
    msg.setContent(message, "text/plain");
    Transport.send(msg);
  }
  

}
