package dsql.parser;

import dsql.exception.*;


import dsql.value.*;

public abstract class ASTSetOperator extends ASTBinaryOperator {

  public ASTSetOperator(int id) {
    super(id);
  }

  public ASTSetOperator(Parser p, int id) {
    super(p, id);
  }

  protected ConstructedTable createNewTable(TableResult t1) 
      throws ExecuteException {
    ConstructedTable tnew = new ConstructedTable(t1);
    if(tnew.getPrimaryKeys()!=null) 
	tnew.setPrimaryKeys(new int[0]); // remove primary keys. not set to
        // null because that would be interpreted as "not set yet"
    tnew.insert(t1);
    return tnew;
  }

  protected ConstructedTableHash createNewTableHash(TableResult t1,
						    TableResult t2) 
      throws ExecuteException {
    ConstructedTableHash tnew = new ConstructedTableHash(t1);
    //    if(t1.getPrimaryKeys()!=null) tnew.setPrimaryKeys(t1.getPrimaryKeys());
    if(tnew.getPrimaryKeys()==null && t2.getPrimaryKeys()!=null)
      tnew.setPrimaryKeys(t2.getPrimaryKeys());
    insertNew(tnew,t1);
    return tnew;
  }

  /** insert only new records of t2 into t1 */
  protected void insertNew(ConstructedTableHash t1, TableResult t2) 
    throws ExecuteException {
    t2.reset();
    while(t2.nextRecord()) {
      RecordHash rechash = t1.getNewRecordHash(t2.readCurrentRecord());
      if(!t1.exists(rechash)) t1.addRecord(rechash);
    }
  }


}
