package dsql.parser;

import dsql.exception.*;
import dsql.value.*;

import java.util.*;
import java.lang.reflect.*;

/** An XML element */
public class ASTXMLElement extends DMNode implements Cloneable {

  public String tagName = null;
  public ArrayList attrs = new ArrayList();

  public ASTXMLElement(int id) {
    super(id);
  }

  public ASTXMLElement(Parser p, int id) {
    super(p, id);
  }

  /** Used for Parser recast */
  public ASTXMLElement(ASTXMLElement toCopy) {
    super(toCopy);
    setTagName(toCopy.getTagName());
    attrs = toCopy.attrs;
  }

  // TBD: should be done with "clone" instead?
  /*public ASTXMLElement(ASTXMLElement toCopy, Node[] _children,
		       ArrayList _attrs) {
    super(toCopy);
    setTagName(toCopy.getTagName());
    attrs = _attrs;
    children = _children;
    }*/

  public String getAttributeValue(String attrName) {
    int i = attrs.indexOf(new XMLAttribute(attrName));
    if(i==-1) 
      return null;
    else 
      return ((XMLAttribute) attrs.get(i)).getValue();
  }

  /** This method adds a new attribute, even if it already existed */
  public void setAttribute(String attrName, String attrVal) {
    attrs.add(new XMLAttribute(attrName,new ASTStaticString(attrVal)));
  }

  public void setAttribute(String attrName, ASTStringExpr attrVal) {
    attrs.add(new XMLAttribute(attrName,attrVal));
  }

  public void setTagName(String newTagName) {
    tagName = newTagName;
  }

  public String getTagName() {return tagName;}

  public ASTXMLElement getElement(Class c) {
    for(int i=0; i<jjtGetNumChildren(); i++)
      if(c.isInstance(children[i]))
        return (ASTXMLElement) children[i];
    return null;
  }
  public ASTXMLElement getElement(String tagName) {
    for(int i=0; i<jjtGetNumChildren(); i++) {
      String _tagName = ((ASTXMLElement)children[i]).getTagName();
      if(_tagName!=null &&_tagName.equals(tagName))
        return (ASTXMLElement) children[i];
    }
    return null;
  }
  public ASTXMLElement[] getElements(Class c) {
    Vector v = new Vector();
    for(int i=0; i<jjtGetNumChildren(); i++)
      if(c.isInstance(children[i]))
        v.add(children[i]);
    ASTXMLElement[] r = (ASTXMLElement[]) Array.newInstance(c,v.size());
    return (ASTXMLElement[]) v.toArray(r);
  }

  public String toString() {return getStartTag();}
  public String getStartTag() {
    return "<"+getTagName()+attributesToString()+">";
  }
  public String attributesToString() {
    if(attrs.isEmpty())
      return "";
    else {
      String r = "";
      for(int i=0; i<attrs.size(); i++)
        r = r+" "+attrs.get(i);
      return r;
    }
  }

  public void print(ASTPrint printer) throws ExecuteException {
    printer.print(getStartTag());
    for(int i=0; i<jjtGetNumChildren(); i++) {
      DMNode child = (DMNode)jjtGetChild(i);
      if(child instanceof ASTXMLElement)
	((ASTXMLElement)child).print(printer);
    }
    printer.print("</"+getTagName()+">");
  }

  /** This returns only the static text; to evaluate
   * DSQL code or variables, call executeXML().getXMLText() */
  public String getXMLText() throws ExecuteException {
    String r = "";
    for(int i=0; i<jjtGetNumChildren(); i++)
      if(children[i] instanceof ASTXMLElement)
        r = r+((ASTXMLElement)children[i]).getXMLText();
    return r;
  }

  public Value execute() throws ExecuteException {
    return new XmlDataValue(executeXML());
  }

  public void setDoc(ASTDoc _doc) {}

  /** Execute DSQL code inside children elements and attributes */
  protected ASTXMLElement executeXML() throws ExecuteException {
    boolean changed = false;
    // attributes
    ArrayList newAttrs = null;
    for(int i=0; i<attrs.size(); i++) {
      XMLAttribute thisAttr = ((XMLAttribute)attrs.get(i)).executeXML();
      if(attrs.get(i) != thisAttr)
	// changed!
	if(newAttrs==null) {
          newAttrs = new ArrayList();
          for(int j=0; j<i; j++)
	    newAttrs.add(attrs.get(j));
        }
      if(newAttrs!=null)
	newAttrs.add(thisAttr);
    }
    // children
    Node[] newChildren = null;
    for(int i=0; i<jjtGetNumChildren(); i++) {
      ASTXMLElement thisChild = ((DMNode)children[i]).executeXML();
      if(children[i] != thisChild)
	// changed!
	if(newChildren==null) {
	  newChildren = new Node[jjtGetNumChildren()];
          for(int j=0; j<i; j++)
	    newChildren[j] = children[j];
	}
      if(newChildren!=null)
	newChildren[i] = thisChild;
    }
    // return value
    if(newChildren!=null || newAttrs!=null) 
      try {
        ASTXMLElement newEle = (ASTXMLElement) clone();
	if(newChildren!=null) newEle.children = newChildren;
	if(newAttrs!=null) newEle.attrs = newAttrs;
	return newEle;
      } catch (CloneNotSupportedException exc) {
	  throw new ExecuteException("unable to copy "+this,exc);
      }
    else 
      return this;
  }

}
