package dsql.value;

import dsql.DSQL;


import dsql.exception.*;


import java.sql.*;
import java.util.*;

/** A ConstructedResults value */
public class ConstructedResults extends AbstractResults {

  ArrayList results = new ArrayList();
  int currentResultIndex = 1; // positioned on first result

  public ConstructedResults(DSQL _caller) {
    setCaller(_caller);
  }

  public boolean hasResult() {
    return currentResultIndex<=results.size();
  }

  public boolean nextResult() {
    currentResultIndex++;
    return hasResult();
  }

  public void allreset() throws ExecuteException {
    currentResultIndex = 1; // positioned on first result
    reset();
  }

  public void reset() throws ExecuteException {
    if(hasResult()) getTable(currentResultIndex).reset();
  }

  public boolean nextRecord() throws ExecuteException {
    return getTable(currentResultIndex).nextRecord();
  }

  public int currentRecord() throws ExecuteException {
    return getTable(currentResultIndex).currentRecord();
  }

  public int getRecordCount() throws ExecuteException {
    return getTable(currentResultIndex).getRecordCount();
  }

  public void gotoRecord(int pos) throws ExecuteException {
    getTable(currentResultIndex).gotoRecord(pos);
  }

  public Object getColumn(int i) throws ExecuteException {
    return getTable(currentResultIndex).getColumn(i);
  }

  public Object getColumn(String s) throws ExecuteException {
    return getTable(currentResultIndex).getColumn(s);
  }

  public String getColumnString(int i) throws ExecuteException {
    return getTable(currentResultIndex).getColumnString(i);
  }

  public String getColumnString(String s) throws ExecuteException {
    return getTable(currentResultIndex).getColumnString(s);
  }

  public int getColumnCount() throws ExecuteException {
    return getTable(currentResultIndex).getColumnCount();
  }

  public String getColumnLabel(int i) throws ExecuteException {
    return getTable(currentResultIndex).getColumnLabel(i);
  }

  public int getColumnDisplaySize(int i) throws ExecuteException {
    return getTable(currentResultIndex).getColumnDisplaySize(i);
  }

  public String[] readCurrentRecord() throws ExecuteException {
    return getTable(currentResultIndex).readCurrentRecord();
  }

  public void addTable(String[] labels) throws ExecuteException {
    results.add(new ConstructedTable(getCaller(),labels));
  }

  public void addRecord(String[] record) throws ExecuteException {
    getTable(lastTable()).addRecord(record);
  }

  // TBD: should return for all tables (instead of just current table)
  public ConstructedResults getMeta(boolean all) throws ExecuteException {
    if(all) throw new ExecuteException("Meta all not implemented for "+
					 "ConstructedResults");
    return getTable(currentResultIndex).getMeta(); 
  }

  public void addCurrentResultMeta(ConstructedResults r)
    throws ExecuteException {
  }

  public void insert(ColumnHolder rec) throws ExecuteException {
    getTable(currentResultIndex).insert(rec);
  }
  public void deleteRecord() throws ExecuteException{
    getTable(currentResultIndex).deleteRecord();
  }
  public void update(ColumnHolder rec) throws ExecuteException {
    getTable(currentResultIndex).update(rec);
  }

  /*********** PROTECTED **********/

  protected int findColumn(String label) throws ExecuteException {
    return getTable(currentResultIndex).findColumn(label);
  }

  protected int lastTable() {return results.size();}

  protected ConstructedTable getTable(int i) throws ExecuteException {
    if(i<=results.size())
      return (ConstructedTable) results.get(i-1);
    else throw new ExecuteException("Attempt to read table past the end "+
                                    "of results!");
  }

/*  protected String[] getRecord(int table, int i) throws ExecuteException {
    return getTable(table).getRecord(i);
  }*/

}
