package dsql.value;

import dsql.DSQL;


import dsql.exception.*;


import java.util.*;

/** A ConstructedResults value */
public class ConstructedTable extends TableResult {

  String id; // just a name to return for toString()
  String[] columnIndex = null; // column labels by index
  int[] columnDisplaySize = null;
  Hashtable columnLabels = new Hashtable();
  ArrayList records = new ArrayList();
  TableResult refTable = null; // the table that was used to create this 
                               // structure

  int currentRecordIndex = 0; // positioned before first record

  public ConstructedTable(DSQL _caller,String[] labels) {
    setCaller(_caller);
    columnIndex = labels;
    for(int i=0; i<labels.length; i++)
      columnLabels.put(labels[i],new Integer(i+1));
  }


  /** Creates a table with the same structure as source */
  public ConstructedTable(TableResult source) throws ExecuteException {
    setCaller(source.getCaller());
    int count = source.getColumnCount();
    columnIndex = new String[count];
    columnDisplaySize = new int[count];
    for(int i=1; i<=count; i++) {
      String label = source.getColumnLabel(i);
      columnIndex[i-1] = label;
      columnLabels.put(label,new Integer(i));
      columnDisplaySize[i-1] = source.getColumnDisplaySize(i);
    }
    refTable = source;
  }

  public String getTableName() {return id;}
  public void setId(String _id) {id=_id;}

  public String toString() {
      if(id==null) return "<data>"; else return "<"+id+">";
  }

  public void insert(TableResult source) throws ExecuteException {
    source.gotoRecord(0);
    while(source.nextRecord()) {
      addRecord(source.readCurrentRecord());
    }
    if(refTable==null) refTable = source;
  }

  public String[] readCurrentRecord() throws ExecuteException {
    return ((String[]) records.get(currentRecordIndex-1));
  }

  public void addRecord(String[] record) throws ExecuteException {
    if(columnIndex.length!=record.length)
      throw new ExecuteException("Can't add record: number of fields "+
                                 "doesn't match columns");
    records.add(record);
  }


  public String getColumnString(int i) throws ExecuteException {
    if(currentRecordIndex<1 || currentRecordIndex>getRecordCount())
      throw new ExecuteException("Record #"+currentRecordIndex+
                                 " outside valid range");
    if(i<1 || i>getColumnCount())
      throw new ExecuteException("Column #"+i+" outside valid range");
    return (readCurrentRecord())[i-1];
  }
  public Object getColumn(int i) throws ExecuteException {
    return getColumnString(i);
  }
  
  public String getColumnString(String s) throws ExecuteException {
    return getColumnString(findColumn(s));
  }
  public Object getColumn(String s) throws ExecuteException {
    return getColumnString(s);
  }

  public String getColumnLabel(int i) {
    return columnIndex[i-1];
  }

  public int getColumnDisplaySize(int k) throws ExecuteException {
    if(columnDisplaySize==null) {
      columnDisplaySize = new int[columnIndex.length];
      for (int i=1; i<=getRecordCount(); i++)
        for (int j=1; j<=getColumnCount(); j++) {
          String value = ((String[]) records.get(i-1))[j-1];
          if(value!=null && value.length()>columnDisplaySize[j-1])
            columnDisplaySize[j-1] = value.length();
        }
    }
    return columnDisplaySize[k-1];
  }

  public int getColumnCount() {return columnIndex.length;}

  public int getRecordCount() {return records.size();}

  public ConstructedResults getMeta() throws ExecuteException {
    ConstructedResults r;
    int[] keys = getPrimaryKeys();
    int keysIndex = 0;
    if(refTable!=null) {
      // if this table was created from another table, get meta from it
      // but use this primary key 
      r = refTable.getMeta();
      r.reset();
      while(r.nextRecord()) {
        String[] colValues = r.readCurrentRecord();
        keysIndex = setMetaLocal(colValues,keys,r.currentRecord(),keysIndex);
      }
    } else {
      r = new ConstructedResults(getCaller());
      r.addTable(METALABELS);
      int n = getColumnCount();
      for(int i=1; i<=n; i++) {
	String[] colValues = new String[8];
	colValues[0] = getColumnLabel(i);
	colValues[1] = "?";
	colValues[2] = "STRING";
	colValues[3] = String.valueOf(getColumnDisplaySize(i));
	colValues[4] = "?"; // precision
	colValues[5] = "?"; // scale
        keysIndex = setMetaLocal(colValues,keys,i,keysIndex);
	r.addRecord(colValues);
      }
    }
    return r;
  }

  protected int setMetaLocal(String[] colValues, int[] keys, 
                               int i, int keysIndex) {
    if(keys==null) colValues[7] = "N";
    else {
      if(keysIndex<keys.length && keys[keysIndex]==i) {
	colValues[7] = "Y";
        keysIndex++;
      } else colValues[7] = "N";
    }
    if(colValues[7].equals("Y")) colValues[6] = "N"; // nullable
    else colValues[6] = "Y";
    return keysIndex;
  }

  public void reset() {
    currentRecordIndex = 0; // positioned before first record
  }

  public boolean nextRecord() throws ExecuteException {
    currentRecordIndex++;
    return currentRecordIndex<=getRecordCount();
  }

  public int currentRecord() throws ExecuteException {
    return currentRecordIndex;
  }

  public void gotoRecord(int pos) throws ExecuteException {
    if(pos==-1) currentRecordIndex=getRecordCount()+1;
    else currentRecordIndex = pos;
  }

  public int[] getPrimaryKeys() throws ExecuteException {
    if(super.getPrimaryKeys()!=null) return super.getPrimaryKeys();
    if(refTable!=null) return setPrimaryKeys(refTable.getPrimaryKeys());
    return null;
  }

  public void insert(ColumnHolder rec) throws ExecuteException {
    addRecord(new String[getColumnCount()]);
    gotoRecord(getRecordCount());
    // TBD: updates!
  }
  public void deleteRecord() throws ExecuteException{
    if(currentRecordIndex<1 || currentRecordIndex>getRecordCount()) 
      throw new ExecuteException("No current record to delete");
    records.remove(currentRecordIndex-1);
  }
  public void update(ColumnHolder rec) throws ExecuteException {
    if(currentRecordIndex<1 || currentRecordIndex>getRecordCount()) 
      throw new ExecuteException("No current record to update");
    // TBD!
  }

  protected void update(int col,Object val) throws ExecuteException {
    if(currentRecordIndex<1 || currentRecordIndex>getRecordCount()) 
      throw new ExecuteException("No current record to update");
    if(col<1 || col>getColumnCount())
      throw new ExecuteException("Column #"+col+" outside valid range");
    ((String[])records.get(currentRecordIndex-1))[col-1] = val.toString();
  }

  /** needed by TableResult.setPrimaryKeys 
   * @return index of label column in table */
  protected int findColumn(String label) throws ExecuteException {
    Integer i = (Integer) columnLabels.get(label);
    if(i!=null) return i.intValue();
    else throw new ExecuteException("No column "+label+" in results");
  }

}
