package dsql.value;

import dsql.DSQL;
import dsql.exception.*;
import dsql.parser.ASTPrint;

/** A pointer to a record in a table. This class implements the
 * DotAt interface, which means its contents can be accessed
 * with ".@n". It also implements the ColumnHolder interface,
 * so that it can be used in Table or set operations whenever
 * a record is expected. */
public class RecordProxy extends NonScalarValue 
    implements DotAt, ColumnHolder {

  TableResult table;
  String name;
  int recordIndex;

  public RecordProxy(TableResult _table, int _recordIndex) 
      throws ExecuteException {
    table = _table;
    setCaller(((Value)table).getCaller());
    recordIndex = _recordIndex;
    name = "Record #"+recordIndex+" from "+table.toString();
  }

  public String toString() {return name;}

  public String getString() throws ExecuteException {
    return getTable().getString();
  }
  public boolean getBoolean() throws ExecuteException {
      return Value.getValue(getCaller(),getField(1)).getBoolean();
  }

  public void print(ASTPrint printer) throws ExecuteException {
    getTable().printRecord(printer);
  }

  /** Access an object's field */
  public Object getField(String name) throws ExecuteException {
    return getColumn(name);
  }
  public Object getField(int i) throws ExecuteException {
    return getColumn(i);
  }
  public Object setField(int i,Value val) throws ExecuteException {
      throw new ExecuteException("RecordProxy.setField(int,Value) not implemented yet");
  }

  public void updateField() throws ExecuteException {
  }

  static final String[] PROXYLABELS = { "Table name", "Index" };

  public ConstructedResults getMeta() throws ExecuteException {
    ConstructedResults r = new ConstructedResults(getCaller());
    r.addTable(PROXYLABELS);
    String[] colValues = new String[2];
    colValues[0] = getTable().toString();
    colValues[1] = ""+recordIndex;
    r.addRecord(colValues);
    return r;
  }

  // ********* DELEGATED methods

  public Object getColumn(int i) throws ExecuteException {
    return getTable().getColumn(i);
  }
  public String getColumnString(int i) throws ExecuteException {
    return getTable().getColumnString(i);
  }

  public Object getColumn(String s) throws ExecuteException {
    getCaller().trace.debug("Fetching column "+s+" record #"+recordIndex);
    return getTable().getColumn(s);
  }
  public String getColumnString(String s) throws ExecuteException {
    getCaller().trace.debug("Fetching column "+s+" record #"+recordIndex);
    return getTable().getColumnString(s);
  }

  public int getColumnCount() throws ExecuteException {
    return getTable().getColumnCount();
  }

  public String getColumnLabel(int i) throws ExecuteException {
    return getTable().getColumnLabel(i);
  }

  public int getColumnDisplaySize(int i) throws ExecuteException {
    return getTable().getColumnDisplaySize(i);
  }

  // ********** PROTECTED *********

  protected TableResult getTable() throws ExecuteException {
    if(table.currentRecord()!=recordIndex)
      table.gotoRecord(recordIndex);
    return table;
  }

}
