package dsql.value;

import dsql.DSQL;
import dsql.exception.*;
import dsql.parser.ASTPrint;

import java.util.*;
import java.sql.*;

/** A table, i.e. a complex value with rows and columns */
public abstract class TableResult extends NonScalarValue
    implements Cursor, ColumnHolder {

  protected int[] primaryKeys = null;

  public String getTableName() {return "table";}

  public int getColumnDisplaySize(int i) throws ExecuteException {
    return 20; // default column size
  }

  /** Access a column */
  public Object getField(String name) throws ExecuteException {
    return getColumn(name);
  }


  public boolean equalsRecord(ColumnHolder r) throws ExecuteException {
    int c = getColumnCount();
    if(r.getColumnCount()!=c) return false;
    for(int i=1; i<=c; i++)
      if(!getColumn(i).equals(r.getColumn(i))) return false;
    return true;
  }

  public void printRecord(ASTPrint printer) throws ExecuteException {
    printer.printTableHeader(this);
    printer.printRecord(this);
    if(printer.getMode()==0 && printer.getHeader()) 
      printer.printTableLine();
  }

  public void print(ASTPrint printer) throws ExecuteException {
    printer.printTable(this);
  }

  /** In a string context, a TableResults returns the first
   * column of the first row of its first result */
  public String getString() throws ExecuteException {
    int currentRec=currentRecord(), colCount=getColumnCount();
    if(colCount==0) return null;
    if(currentRec>0) return getColumnString(1);
    else {
      if(nextRecord()) {
        String res=getColumnString(1); reset(); return res;
      } else return null;
    }
  }

  /** @return true if has some records */
  public boolean getBoolean() throws ExecuteException {
      if(currentRecord()>0) return true;
      else {
	  if(nextRecord()) {reset(); return true;}
	  else return false;}
  }

  public int[] getPrimaryKeys() throws ExecuteException {return primaryKeys;}

  public int[] setPrimaryKeys(String[] keysStr) throws ExecuteException {
    if(keysStr==null) return null;
    int[] keys = new int[keysStr.length];
    for(int i=0; i<keys.length; i++)
      keys[i] = findColumn(keysStr[i]);
    Arrays.sort(keys);
    return setPrimaryKeys(keys);
  }

  public int[] setPrimaryKeys(int[] _keys) {
    primaryKeys = _keys;
    return primaryKeys;
  }

  protected abstract int findColumn(String label) throws ExecuteException;

  public abstract ConstructedResults getMeta() throws ExecuteException;

  public abstract void insert(ColumnHolder rec) throws ExecuteException;
  public abstract void deleteRecord() throws ExecuteException;
  public abstract void update(ColumnHolder rec) throws ExecuteException;

  public Value getRecord() throws ExecuteException {
    return getRecord(currentRecord());
  }
  public Value getRecord(int i) throws ExecuteException {
    return new RecordProxy(this,i);
  }

  public void setRecord(int i, Value val) throws ExecuteException {
      throw new ExecuteException("Table results cannot be modified (not implemented yet)");
  }


  public abstract String[] readCurrentRecord() throws ExecuteException;
}
