package dsql.value;

import dsql.DSQL;
import dsql.exception.*;
import dsql.parser.*;

import java.io.*;

public class XmlRecordHolder extends XmlDataValue implements Cursor {

  protected int currentRecordIndex = 0;
  protected int currentRecordPos = -1;

  // TBD: put file pointer to same position as data 
  public XmlRecordHolder(XmlDataValue data) throws ExecuteException {
    super();
    data.getCaller().trace.debug("Creating new XML record holder...");
    if(data.external)
      setContents(data.fileName,data.dataEle);
    else
      setContents(data.dataEle);
    readColumnFromFile = false;
  }

  /** Parse next column and add it to buffer */
  protected ASTXMLElement parseNextRecord() throws ExecuteException {
    initParser();
    while((buffer=parseNext())!=null && buffer.getTagName()==null) {}
    return buffer;
  }

  // ********************** RecordHolder methods

  public void reset() throws ExecuteException {
    gotoRecord(0);
  }

  public int currentRecord() throws ExecuteException {
    return currentRecordIndex;
  }

  public void gotoRecord(int pos) throws ExecuteException {
    if(pos<currentRecordIndex)
      if(external) 
        throw new ExecuteException("Can't reset XmlRecordHolder yet");
      else {
	currentRecordIndex = 0;
	currentRecordPos = -1;
      }
    while(pos>currentRecordIndex && nextRecord()) {}
  }

  public boolean nextRecord() throws ExecuteException {
    currentRecordIndex++;
    if(external) {
      return parseNextRecord()!=null;
    } else
      throw new ExecuteException("XmlRecordHolder.nextRecord() not implemented for inline!");
  }

  public int getRecordCount() throws ExecuteException {
      throw new ExecuteException("XmlRecordHolder.getRecordCount() not implemented");
  }

  public Value getRecord() throws ExecuteException {
    return getRecord(currentRecord());
  }
  public Value getRecord(int i) throws ExecuteException {
    gotoRecord(i);
    return new XmlDataValue(getXmlData());
  }

  public void printRecord(ASTPrint printer) throws ExecuteException {
      // TBD: shouldn't it try to load the buffer if it's null?
    if(buffer!=null) {
      buffer.print(printer);
    }
  }

  public void setRecord(int i, Value val) throws ExecuteException {
      throw new ExecuteException("XML index cannot be modified (not implemented yet)");
  }

}

